@@ -48,6 +48,7 @@ LevelDaten * ld;
 /** */
 LevelDaten::LevelDaten(const Version & version): mLevelCache(),
   mVersion(version),
+  mLevelGeladen(false),
 /* Wir basteln uns ein Array, dessen Indizierung nicht mit 0 beginnt...: */
   mSorten(mSortenIntern - blopart_min_sorte),
   /* Noch keine Sorten geladen */
@@ -58,14 +59,17 @@ LevelDaten::LevelDaten(const Version & version): mLevelCache(),
 {
   for (int teil=0; teil<ldteile_anzahl; teil++) {
     mLevelConf[teil] = new DatenDatei();
-    mGeladen[teil] = false;
+    mLCGeladen[teil] = false;
   }
   ld = this;
-  ladLevelConf(ldteil_summary,true,version);
   
   /* Auch die negativen Sorten sind noch nicht geladen: */
   for (int i = blopart_min_sorte; i < 0; i++)
     mSorten[i] = NULL;
+
+  /* So; jetzt haben die Variablen einen gltigen Zustand,
+     und wir knnen schon mal erste Dinge laden. */
+  ladLevelConf(ldteil_summary,true,version);
 }
 
 
@@ -84,13 +88,19 @@ void LevelDaten::ladLevelConf(LDTeil ldteil, bool aufJedenFall,
 
   if (aufJedenFall || (version!=mVersion)) {
   
+    /* Wenn noch ein Level geladen war, dann wird der jetzt ungltig.
+       Damit nicht jemand auf die Idee kommt, den nochmal zu verwenden,
+       lieber entladen. */
+    entladLevel();
+    
+  
     mVersion = version;
 
     mSpielerZahl = (mVersion.enthaelt("1") ? 1 : 2);
 
     /* Jetzt ist erst mal nix mehr geladen. Erst wenn ladLevelConf()
        durchgelaufen ist, ist wieder was da. */
-    mGeladen[ldteil] = false;
+    mLCGeladen[ldteil] = false;
 
     /* Abkrzung */
     DatenDatei * conf = mLevelConf[ldteil];
@@ -147,7 +157,7 @@ void LevelDaten::ladLevelConf(LDTeil ldteil, bool aufJedenFall,
 
 	  /* Hier findet schon wieder parsen statt. */
 	  mSammleLevel = true;
-	  conf->laden(gLevelDatei);
+	  conf->laden(vergissPfad(gLevelDatei));
 
 	} else {
 
@@ -175,17 +185,18 @@ void LevelDaten::ladLevelConf(LDTeil ldteil, bool aufJedenFall,
 	  }
 	}
 
-	mGeladen[ldteil] = true;
+	mLCGeladen[ldteil] = true;
 
 	break;
       case ldteil_level:
-	CASSERT(mGeladen[ldteil_summary]);
+	CASSERT(mLCGeladen[ldteil_summary]);
 	{
 	  DatenDateiPush ddp(*(mLevelConf[ldteil_summary]),
 			     mIntLevelName, mVersion);
 	  Str datei = (gDateiUebergeben
             ? mLevelConf[ldteil_summary]->
-	        getWortEintragMitDefault("filename",mVersion,gLevelDatei)
+	        getWortEintragMitDefault("filename",mVersion,
+					 vergissPfad(gLevelDatei))
 	    : mLevelConf[ldteil_summary]->
                 getWortEintragOhneDefault("filename",mVersion));
 	  if (mLevelCache.find(datei)==mLevelCache.end()) {
@@ -205,7 +216,7 @@ void LevelDaten::ladLevelConf(LDTeil ldteil, bool aufJedenFall,
     }
 
     /* OK, Laden war erfolgreich. */
-    mGeladen[ldteil] = true;
+    mLCGeladen[ldteil] = true;
   }
 }
 
@@ -232,8 +243,6 @@ void LevelDaten::ladSorten(const Str & ldKeyWort, int blopart) {
   if (picsnamen) {
     int neueNamen = picsnamen->getLaenge();
     int neueFarben = picsnamen->getImpliziteLaenge();
-    mAnzFarben += neueFarben;
-
     /** Die Nummern der Sorten wurden schon beim parsen in knoten.cpp
         festgelegt. Hier tun wir unser bestes, die selben Nummern zu
 	bekommen. nr ist die, die zum nchsten logischen Listeneintrag
@@ -241,13 +250,17 @@ void LevelDaten::ladSorten(const Str & ldKeyWort, int blopart) {
     int nr = mLevelConf[ldteil_level]->getSquirrelPos()->
       getSortenAnfang(ldKeyWort);
     if (nr+neueFarben > max_farben_zahl)
-      throw Fehler(_sprintf(_("#pics > %d"), max_farben_zahl));
+      /* TRANSLATORS: "pics" is a programming keyword and should not be
+	 translated */
+      throw Fehler(_sprintf(_("Number of pics exceeds limit %d"),
+			    max_farben_zahl));
 
     for (int namen_nr = 0; namen_nr < neueNamen; namen_nr++) {
       mSorten[nr] = new Sorte(picsnamen->getKernDatum(namen_nr,type_WortDatum)
 		                ->getWort(),
 			      mVersion, blopart);
       nr++;
+ 
       for (int i = picsnamen->getVielfachheit(namen_nr)-1; i>0; i--) {
 	mSorten[nr]=mSorten[nr-1];
 	nr++;
@@ -260,12 +273,19 @@ void LevelDaten::ladSorten(const Str & ldKeyWort, int blopart) {
 
 /* Gibt Speicher frei */
 void LevelDaten::entladLevel() {
+  mLevelGeladen = false;
+  
+  /* Wir prfen vor dem Freigeben nicht, mLevelGeladen = true war:
+     vielleicht war aufgrund eines Fehlers irgendwas in einem
+     halbgeladenen Zustand; dann wrden wir auch gerne allen
+     Speicher freigeben, der grad belegt war */
+  
   /* Positive Sorten lschen */
   for (int bnr=mAnzFarben-1; bnr>0; bnr--)
-    if (mSorten[bnr]!=mSorten[bnr-1])
-      delete mSorten[bnr];
-  if (mAnzFarben>0)
-    delete mSorten[0];
+    if (bnr == 0 || mSorten[bnr]!=mSorten[bnr-1]) {
+      if (mSorten[bnr])  // Koennte NULL sein falls ladefehler
+        delete mSorten[bnr];
+    }
   mAnzFarben = 0;
 
   /* Die negativen Sorten lschen */
@@ -281,7 +301,7 @@ void LevelDaten::entladLevel() {
 /** fllt alle Daten in diesem Objekt fr Level nr aus; throwt bei Fehler */
 void LevelDaten::ladLevel(int nr) {
 
-  if (!mGeladen[ldteil_summary])
+  if (!mLCGeladen[ldteil_summary])
     throw Fehler(_("Sorry, no working level description file available."));
 
   /* Ggf. Speicher von altem Level freigeben */
@@ -301,7 +321,7 @@ void LevelDaten::ladLevel(int nr) {
 
   ladLevelConf(ldteil_level, true, mVersion);
 
-  if (!mGeladen[ldteil_level])
+  if (!mLCGeladen[ldteil_level])
     throw Fehler(_("Sorry, no working level description file available."));
 
   try {
@@ -310,27 +330,29 @@ void LevelDaten::ladLevel(int nr) {
        bei } wieder raus. */
     DatenDateiPush ddp(*(mLevelConf[ldteil_level]), mIntLevelName, mVersion);
 
-    fehlerpos = mLevelConf[ldteil_level]->getSquirrelPos()->getDefString() +
-                _(" (or somewhere below): ");
+    DatenDatei * daten = mLevelConf[ldteil_level];
+
+    // TRANSLATORS: This prepends an error message with position information
+    fehlerpos = _sprintf(_("At position %s (or somewhere below): "),
+			 daten->getSquirrelPos()->getDefString().data());
 
     /* DefKnoten des Levels abspeichern. */
-    mLevelKnoten = mLevelConf[ldteil_level]->getSquirrelPos();
+    mLevelKnoten = daten->getSquirrelPos();
 
     /* Level-Name */
-    mLevelName = mLevelConf[ldteil_level]->
-      getWortEintragOhneDefault("name",mVersion);
+    mLevelName = _(daten->getWortEintragOhneDefault("name",mVersion).data());
 
     /* Level-Autor */
-    mLevelAutor = mLevelConf[ldteil_level]->
-      getWortEintragOhneDefault("author",mVersion);
+    mLevelAutor = _(daten->
+      getWortEintragOhneDefault("author",mVersion).data());
 
     /* Beschreibungstext (optional) */
-    mBeschreibung = mLevelConf[ldteil_level]->
-      getWortEintragMitDefault("description", mVersion, "");
+    mBeschreibung = _(daten->
+      getWortEintragMitDefault("description", mVersion, "").data());
 
     /* Wie viele Steine mssen zusammen, damit sie platzen?
        (optional, da je Sorte definierbar (mu man dann aber auch tun)) */
-    mPlatzAnzahlDefault = mLevelConf[ldteil_level]->getZahlEintragMitDefault
+    mPlatzAnzahlDefault = daten->getZahlEintragMitDefault
       ("numexplode", mVersion, PlatzAnzahl_undefiniert);
     mPlatzAnzahlMin = PlatzAnzahl_undefiniert;
     mPlatzAnzahlMax = PlatzAnzahl_undefiniert;
@@ -340,64 +362,62 @@ void LevelDaten::ladLevel(int nr) {
        Achtung: Die Hintergrundfarbe muss gesetzt werden, _bevor_
        Bildchen geladen werden, da es als Bonus-Farbe im XPM
        "Background" gibt... (im Moment nur fr Explosion sinnvoll) */
-    mHintergrundFarbe = mLevelConf[ldteil_level]->
+    mHintergrundFarbe = daten->
       getFarbEintragMitDefault("bgcolor", mVersion, Color(255, 255, 255));
 
     /* Hintergrundbilchen (optional) */
-    mMitHintergrundbildchen = mLevelConf[ldteil_level]->hatEintrag("bgpic");
+    mMitHintergrundbildchen = daten->hatEintrag("bgpic");
     if (mMitHintergrundbildchen)
-      mHintergrundBild.laden(mLevelConf[ldteil_level]->
+      mHintergrundBild.laden(daten->
 			     getWortEintragOhneDefault("bgpic", mVersion));
 
     /* Schriftfarbe... (optional; Default: dunkelgrau) */
-    setSchriftFarbe(mLevelConf[ldteil_level]->
-		    getFarbEintragMitDefault("textcolor",mVersion,
-					     Color(40, 40, 40)));
+    mSchriftFarbe = daten->
+      getFarbEintragMitDefault("textcolor",mVersion,Color(60, 60, 60));
+    setSchriftFarbe(mSchriftFarbe);
 
     /* Hetzrandfarbe... (optional; Default: hellgrau) */
-    hetzrandFarbe = mLevelConf[ldteil_level]->
+    hetzrandFarbe = daten->
       getFarbEintragMitDefault("topcolor", mVersion, Color(200, 200, 200));
 
     /* Hetzrandgeschwindigkeit (optional) */
-    hetzrandZeit = mLevelConf[ldteil_level]->
+    hetzrandZeit = daten->
       getZahlEintragMitDefault("toptime", mVersion, toptime_default);
     if (hetzrandZeit < 1)
-      throw Fehler(_("toptime < 1"));
+      /* TRANSLATORS: "toptime" is a programming keyword which should not
+	 be translated. */
+      throw Fehler(_("toptime must be positive"));
 
     /* Hetzrandbildchen (optional) */
-    mMitHetzbildchen = mLevelConf[ldteil_level]->hatEintrag("toppic");
+    mMitHetzbildchen = daten->hatEintrag("toppic");
     if (mMitHetzbildchen) {
-      mHetzBild.laden(mLevelConf[ldteil_level]->
-		      getWortEintragOhneDefault("toppic",mVersion));
+      mHetzBild.laden(daten->getWortEintragOhneDefault("toppic",mVersion));
   	
       /* Hetzrandberlapp (optional) */
-      mHetzrandUeberlapp = mLevelConf[ldteil_level]->
+      mHetzrandUeberlapp = daten->
 	getZahlEintragMitDefault("topoverlap", mVersion, mHetzBild.getHoehe());
     } else
       mHetzrandUeberlapp = 0;
 
-    mHetzrandStop = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("topstop", mVersion, 0);
-
-    mFall_langsam_pix = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("falling_speed", mVersion, 6);
-    mFall_schnell_pix = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("falling_fast_speed", mVersion, gric);
-    if (mFall_schnell_pix<=0)
-      throw Fehler(_("falling_fast_speed must be positive"));
+    mHetzrandStop = daten->getZahlEintragMitDefault("topstop", mVersion, 0);
 
     /* Gras nur bei Kettenreaktion? (optional) */
-    mGrasBeiKettenreaktion = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("chaingrass", mVersion, 0);
+    mGrasBeiKettenreaktion = daten->
+      getBoolEintragMitDefault("chaingrass", mVersion, false);
 
     /* Senkrecht spiegeln? (optional) */
-    mSpiegeln = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("mirror",mVersion,0);
+    mSpiegeln = daten->getBoolEintragMitDefault("mirror",mVersion,false);
+
+    /* Neuer fall wird (horizontal) zufllig plaziert? (optional) */
+    mFallPosZufaellig = daten->
+      getBoolEintragMitDefault("randomfallpos", mVersion, false);
 
     /* Andere Nachbarschaft? (optional) */
-    mNachbarschaft = mLevelConf[ldteil_level]->
+    mNachbarschaft = daten->
       getZahlEintragMitDefault("neighbours", mVersion, nachbarschaft_normal);
     if (mNachbarschaft < 0 || mNachbarschaft > nachbarschaft_letzte)
+      /* TRANSLATORS: "neighbours" is a programming keyword which
+	 should not be translated. */
       throw Fehler(_("neighbours out of range"));
     /* Sechseck-Raster? */
     mSechseck =
@@ -407,19 +427,20 @@ void LevelDaten::ladLevel(int nr) {
 
 
     /* Zufllige Graue? (optional) */
-    mZufallsGraue = mLevelConf[ldteil_level]->
+    mZufallsGraue = daten->
       getZahlEintragMitDefault("randomgreys", mVersion, zufallsgraue_keine);
   	  	
     /* Wo sind welche Grasbildchen am Anfang? */
-    mAnfangsZeilen = mLevelConf[ldteil_level]->
-      getListenEintrag("startdist",mVersion,false);
+    mAnfangsZeilen = daten->getListenEintrag("startdist",mVersion,false);
 
 
     /***** Noch ein paar einzelne Bilder laden *****/
     
     /* Explosion laden. Das darf erst nach dem Laden der Hintergrundfarbe
-       passierren. */
-    mExplosionBild.laden("explosion.xpm");
+       passieren. */
+    mExplosionBild.laden(daten->
+			 getWortEintragMitDefault("explosionpic",mVersion,
+						  "explosion.xpm"));
 
     /* Schriftfarbe der Punkte neu setzen. (Provisorisch) */
     Punktefeld::init();
@@ -429,7 +450,18 @@ void LevelDaten::ladLevel(int nr) {
 
     /***** Blops laden *****/
 
-    mAnzFarben = 0;
+    /* Falls waehrend des Ladevorgangs ein Fehler auftritt, muessen
+       alle noch ungeladenen Sorten NULL sein.
+       Eigentlich waere es ja am einfachsten, wenn mAnzFarben immer
+       die bisherige Anzahl der geladenen Sorten angibt. Das geht
+       aber nicht, da ladSorten() die Sorten evtl. nicht der Reihe
+       nach laedt. Das liegt daran, dass die Sortennummern nicht
+       in der Reihenfolge vergeben werden, wie hier ladSorten auf-
+       gerufen wird, sondern in der Reihenfolge, wie's in der
+       ld-Datei steht. */
+    mAnzFarben = daten->getSquirrelPos()->getSortenAnzahl();
+    for (int i = 0; i < mAnzFarben; i++)
+      mSorten[i] = NULL;
 
     /* Wie viele Bilder malt ein Blop in einem Schritt hchstens?
        Erst mal keine. Die Sorten erhhen diese Variablen selbst, wenn
@@ -442,11 +474,11 @@ void LevelDaten::ladLevel(int nr) {
     ladSorten("greypic",blopart_grau);
 
     /* Leer-Bildchen (optional) */
-    mMitLeerBildchen = mLevelConf[ldteil_level]->hatEintrag("emptypic");
+    mMitLeerBildchen = daten->hatEintrag("emptypic");
     /* Auch, wenn es kein Leer-Bildchen gibt, soll es geladen werden;
        dann liefert getEintrag() "" zurck, und Sorte::laden() wei,
        dass es nur alle Werte auf Defaults setzen soll. */
-    mSorten[blopart_keins] = new Sorte(mLevelConf[ldteil_level]->
+    mSorten[blopart_keins] = new Sorte(daten->
         getWortEintragMitDefault("emptypic", mVersion, ""),
       mVersion, blopart_keins);
 
@@ -468,9 +500,10 @@ void LevelDaten::ladLevel(int nr) {
       mDistKeyLen=1;
 
     /* Entstehungswahrscheinlichkeiten */
-    mKeineGrauenW = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("nogreyprob", mVersion, 0);
+    mKeineGrauenW = daten->getZahlEintragMitDefault("nogreyprob", mVersion, 0);
     if (mKeineGrauenW<0)
+      /* TRANSLATORS: "nogreyprob" is a programming keyword which
+	 should not be translated. */
       throw Fehler(_("nogreyprob must not be negative"));
     for (int i=0; i<anzahl_wv; i++) {
       mVerteilungSumme[i]=0;
@@ -478,29 +511,30 @@ void LevelDaten::ladLevel(int nr) {
 	mVerteilungSumme[i] += mSorten[j]->getVerteilung(i);
     }
     if (mVerteilungSumme[wv_farbe]==0)
+      /* TRANSLATORS: Don't worry about the %s, it will be a programming
+	 keyword. */
       throw Fehler(_("At least one %s must be positive."),
 		   cVerteilungsNamen[wv_farbe]);
     if (mVerteilungSumme[wv_grau]+mKeineGrauenW==0)
+      /* TRANSLATORS: "nogreyprob" is a programming keyword which
+	 should not be translated.
+         Also, don't worry about the %s, it will be a programming keyword. */
       throw Fehler(_("nogreyprob or at least one %s must be positive."),
 		   cVerteilungsNamen[wv_grau]);
 
 
    /* Musik (optional) */
-   mMusik = mLevelConf[ldteil_level]->
-     getWortEintragMitDefault("music", mVersion, "");
+   mMusik = daten->getWortEintragMitDefault("music", mVersion, "");
 
   /***** KI-Player-Bewertungen *****/
-    mKINHoehe = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("aiu_height", mVersion, 10);
-    mKINAnFarbe = mLevelConf[ldteil_level]->
+    mKINHoehe = daten->getZahlEintragMitDefault("aiu_height", mVersion, 10);
+    mKINAnFarbe = daten->
       getZahlEintragMitDefault("aiu_color", mVersion, 10 * mAnzFarben);
-    mKINAnGras = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("aiu_grass", mVersion, 20);
-    mKINAnGrau = mLevelConf[ldteil_level]->
-      getZahlEintragMitDefault("aiu_grey", mVersion, 10);
-    mKINZweiUeber = mLevelConf[ldteil_level]->
+    mKINAnGras = daten->getZahlEintragMitDefault("aiu_grass", mVersion, 20);
+    mKINAnGrau = daten->getZahlEintragMitDefault("aiu_grey", mVersion, 10);
+    mKINZweiUeber = daten->
       getZahlEintragMitDefault("aiu_two_above", mVersion, mKINAnFarbe / 2);
-    mKINEinfarbigSenkrecht = mLevelConf[ldteil_level]->
+    mKINEinfarbigSenkrecht = daten->
       getZahlEintragMitDefault("aiu_monochromic_vertical", mVersion,
 			       mKINAnFarbe);
 
@@ -512,13 +546,21 @@ void LevelDaten::ladLevel(int nr) {
     fs += f.getText() + "\n";
     
     if (!mLevelName.isEmpty())
-      fs += _("(Level \"") + mLevelName + _("\")\n");
+      // TRANSLATORS: This postpends an error message with some information
+      fs += _sprintf(_(" (Level \"%s\")\n"), mLevelName.data());
     throw Fehler(fs);
   }
-	
+  
+  mLevelGeladen = true;
+  mLevelNummer = nr;
 } // ladLevel
 
 
+void LevelDaten::erneuerLevel(int nr) {
+  CASSERT(mLevelGeladen);
+  CASSERT(nr == mLevelNummer);
+  setSchriftFarbe(mSchriftFarbe);
+}
 
 
 
@@ -582,11 +624,12 @@ Str LevelDaten::getLevelName(int nr) const {
     DatenDateiPush ddp(*(mLevelConf[ldteil_summary]),
 		       getIntLevelName(nr), mVersion);
 
-    return mLevelConf[ldteil_summary]->
-      getWortEintragOhneDefault("name", mVersion);
+    return _(mLevelConf[ldteil_summary]->
+      getWortEintragOhneDefault("name", mVersion).data());
 
   } catch (Fehler f) {
-    return "???";
+    // TRANSLATORS: This indicates an unknown value
+    return _("???");
   }
 }
 
@@ -645,6 +688,8 @@ int LevelDaten::liesDistKey(const Str & key) {
     mDistKeyLen=key.length();
   else
     if (mDistKeyLen!=key.length())
+      /* TRANSLATORS: "distkey" is a programming keyword which should not
+	 be translated */
       throw Fehler(_("distkey \"%s\" does not have length %d as others do."),
 		   key.data(), mDistKeyLen);
 
@@ -672,11 +717,15 @@ int LevelDaten::liesDistKey(const Str & key) {
 	    if (anfang && c==' ')
 	      immernochanfang = true;
 	    else
+	      /* TRANSLATORS: "startdist" and "distkey" are programming keyword
+		 which should not be translated */
 	      throw Fehler(_("Illegal character \"%c\" in startdist or distkey %s"),
 			   c,key.data());
       anfang = immernochanfang;
     }
     if (anfang)
+      /* TRANSLATORS: "startdist" and "distkey" are programming keyword
+	 which should not be translated */
       throw Fehler(_("All-spaces startdist entry or distkey is not allowed."));
     return n;
   }
@@ -690,17 +739,9 @@ const Version & LevelDaten::getVersion() const {
 
 
 
- /** Setzt mSchriftFarbe[...]. Berechnet also insbesondere die dunkle
-     und die helle Farbe. */
+/** Setzt AutoColor::gGame */
 void LevelDaten::setSchriftFarbe(Color f) {
-  mSchriftFarbe[schrift_dunkel] =
-     Color(f.mR / 2, f.mG / 2, f.mB / 2);
-  mSchriftFarbe[schrift_normal] = f;
-  mSchriftFarbe[schrift_hell] =
-     Color(128 + f.mR / 2, 128 + f.mG / 2, 128 + f.mB / 2);
-
-  /* Schriftfarbe der Punkte neu setzen. (Provisorisch) */
-  Punktefeld::init();
-  /* Und Hauptschriftfarbe setzen (Auch irgendwie provisorisch) */
-  Font::setGameColor(f);
+  /* Changing AutoColor::gGame automatically recolors the game font
+     and some other pictures */
+  AutoColor::gGame = f;
 }
