@@ -26,13 +26,12 @@
 #include "cuyointl.h"
 #include "global.h"
 
-
-#define pfad_default 0
-#define pfad_uebergeben 1
-#define pfad_aktueller 2
-#define pfad_uninstalliert 3
-#define pfad_installation 4
-#define anz_pfade 5
+enum {
+  pfad_default,
+  pfad_uebergeben,
+  pfad_installation,
+  anz_pfade
+};
 
 
 int PfadIterator::gDefaultPfad = -1;
@@ -48,6 +47,7 @@ PfadIterator::PfadIterator(Str dat, bool auch_gz /*= false*/,
      -1 und fhren dann gleich ein ++ aus. */
   ++(*this);
 }
+
 PfadIterator::~PfadIterator() {
   /** Soll der default-Pfad gesetzt werden? */
   if (mSetzDefault) {
@@ -128,14 +128,6 @@ Str PfadIterator::pfad() const {
 #endif
       break;
 
-    case pfad_uninstalliert:
-      r = _sprintf("%s../data/%s", gCuyoPfad.data(), mDatei.data());
-      break;
-
-    case pfad_aktueller:
-      r = _sprintf("./%s", mDatei.data());
-      break;
-
     default:
       CASSERT(false);
   }
